// Written by TeTeT
// radar server script
// create radar information when there are subscribers and push it to registered clients

// note for the start we use a publicVariable for all clients ...

if (!isServer) exitWith {};

// Follow a pattern as outlined by deadfast and Muzzleflash in a forum thread (134888-getting-the-clientid-for-publicvariableclient)
// need to get the clientId and store it somewhere
TTT_fnc_subscribeToRadar = {
	systemChat "Server subscription event";
	params ["_n", "_player"];
	private ["_clientId", "_subscribers"];
	_clientId = owner _player;
	_subscribers = TTT_radarCtrl getVariable ["TTT_radarSubscribers", []];
	_subscribers pushBackUnique _clientId;
	TTT_radarCtrl setVariable ["TTT_radarSubscribers", _subscribers];
	_player;
};

"TTT_PV_subscribeToRadar" addPublicVariableEventHandler TTT_fnc_subscribeToRadar;

TTT_fnc_unsubscribeFromRadar = {
	params ["_n", "_player"];
	private ["_clientId", "_subscribers"];
	_clientId = owner _player;
	_subscribers = TTT_radarCtrl getVariable ["TTT_radarSubscribers", []];
	_subscribers = _subscribers - [ _clientId ];
	TTT_radarCtrl setVariable ["TTT_radarSubscribers", _subscribers];
	_player;
};

"TTT_PV_unsubscribeFromRadar" addPublicVariableEventHandler TTT_fnc_unsubscribeFromRadar;

// Get the objects tracked by radar
_fnc_getTrackedObjects = {
   params ["_radars"];

   private ["_allShortRangeObjects", "_allLongRangeObjects", "_radar", "_short", "_long", "_shortRange", "_longRange"];

   _allShortRangeObjects = [];
   _allLongRangeObjects = [];
   {
      _radar = _x;
      _shortRange = _radar getVariable ["TTT_radarShortRange", 1000];
      _longRange = _radar getVariable ["TTT_radarLongRange", 4000];

      _short = [];
      _long = [];
      {
         if ( !isTouchingGround _x and !terrainIntersectASL [getPosASL _radar, getPosASL _x] ) then {
            if (_radar distance _x <= _shortRange) then {
               _short pushBack _x;
            } else {
               _long pushBack _x;
            };
         };
      } forEach ( (position _radar) nearEntities [["Air"], _longRange] );
      {
         _allShortRangeObjects pushBackUnique _x;
      } forEach _short;
      {
         _allLongRangeObjects pushBackUnique _x;
      } forEach _long;
   } forEach _radars;
   // remove all short range objects from long range objects, always show best info
   _allLongRangeObjects = _allLongRangeObjects - _allShortRangeObjects;
   [_allShortRangeObjects, _allLongRangeObjects];
};

TTT_fnc_createAndPushData = {
	private ["_subscribers", "_radars"];
	_subscribers = TTT_radarCtrl getVariable ["TTT_radarSubscribers", []];
	_radars = TTT_radarCtrl getVariable ["TTT_radars", []];

	TTT_radarAllObjects = [_radars] call _fnc_getTrackedObjects;
	{
		systemChat "Server pushing data";
		_x publicVariableClient "TTT_radarAllObjects";
	} forEach _subscribers;
};

while {alive TTT_radarCtrl} do {
	// check for subscribers
	// create data when subscriber is present
	systemChat "Server checking subscribers";
	if (count (TTT_radarCtrl getVariable ["TTT_radarSubscribers", []]) > 0) then {
		systemChat "Server found subscriber";
		[] call TTT_fnc_createAndPushData;
	};
	// moderate sleep time
	sleep 10;
};